// ---------------------------------------------------------------------------
// pda_dance.cs -- Version 1.5 -- October 28, 2000
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// Press a key to perform a random "evasive maneuvers" dance
//
// Useful while you're using the PDA, or if you need to head to leave the
// computer for a bit, or if you want your opponents to think you've flipped. :)
//
// It's great for keeping [SS-19]FlyChic amused, too. :)
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\pda_dance.cs", "1.5", "Lorne Laliberte", "Oct 28, 2000");

include("writer\\save.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_pda_dance.cs file

    // Start or stop dancin' :)
    //bind(keyboard0, make, alt, "d", to, "pdaDance();");
    //bind(keyboard0, break, alt, "d", to, "", "pdaDance_break");

    // Start dancin' and bring up the commander screen
    // (or stop dancin' and close the commander screen)
    //bind(keyboard0, make, alt, "c", TO, "pdaDance::commandScreen();");
    //bind(keyboard0, break, alt, "c", TO, "", pdaDance::commandScreen_break);

// ---------------------------------------------------------------------------

WriterPrefs::load("pda_dance");

// These are the steps our dancer can perform
//
function pdaDance::Step8(%move) // N
{
    postAction(2048, IDACTION_MOVEFORWARD, %move);
}
function pdaDance::Step9(%move) // NE
{
    postAction(2048, IDACTION_MOVEFORWARD, %move);
    postAction(2048, IDACTION_MOVERIGHT, %move);
}
function pdaDance::Step6(%move) // E
{
    postAction(2048, IDACTION_MOVERIGHT, %move);
}
function pdaDance::Step3(%move) // SE
{
    postAction(2048, IDACTION_MOVEBACK, %move);
    postAction(2048, IDACTION_MOVERIGHT, %move);
}
function pdaDance::Step2(%move) // S
{
    postAction(2048, IDACTION_MOVEBACK, %move);
}
function pdaDance::Step1(%move) // SW
{
    postAction(2048, IDACTION_MOVEBACK, %move);
    postAction(2048, IDACTION_MOVELEFT, %move);
}
function pdaDance::Step4(%move) // W
{
    postAction(2048, IDACTION_MOVELEFT, %move);
}
function pdaDance::Step7(%move) // NW
{
    postAction(2048, IDACTION_MOVEFORWARD, %move);
    postAction(2048, IDACTION_MOVELEFT, %move);
}
function pdaDance::Step5(%move) // Duck
{
    if(%move)
        postAction(2048, IDACTION_CROUCH, 0);
    else
        postAction(2048, IDACTION_STAND, 0);
}


// Translate a step to the step that will "retrace" it
//
function pdaDance::getRetraceStep(%step)
{
    if( (%index = String::findSubStr("89632147", %step)) != -1 )
        return String::getSubStr("21478963", %index, 1);
    else
        return %step;
}


// Toggle the dance and the command screen on/off
//
function pdaDance::commandScreen()
{
    remoteEval(2048, ToggleCommandMode);
    pdaDance();
}


// Toggle the dance on/off
//
function pdaDance()
{
    if($pdaDancing)
    {
        $pdaDancing = 0;
    }
    else
    {
        $pdaDancing = 1;
        pdaDance::dance(1, 1); // start dancing "forward" and do init
    }
}


// Dance away in the pda
//
function pdaDance::dance(%forward, %init)
{
    if($pdaDancing)
    {
        if(%forward) // not retracing our footsteps
        {
            if(%init)
            {
                $pdaDance::stepcounter = 0;

                // Set number of steps for this part of the dance
                $pdaDance::steps = 3 + floor(getRandom() * 9); // between 3 and 12
            }

            if($pdaDance::stepcounter < $pdaDance::steps)
            {
                // Still doing the "forward" steps

                // Pick dance step at random (between 1 and 9)
                %step = $pdaDance::stepsTaken[$pdaDance::stepcounter] =  1 + floor(getRandom() * 8);

                // Start doing the step
                eval("pdaDance::Step" @ %step @ "(1);");//, 0);

                // Set a duration for this step between 0.25 and 1.25 seconds
                // and remember it in our $pdaDance::intervals array
                %interval = ($pdaDance::intervals[$pdaDance::stepcounter] = 0.25 + getRandom());

                // Stop this step after our interval has passed
                schedule("pdaDance::Step" @ %step @ "(0);", %interval);

                // Do another step next time (or start reversing)
                $pdaDance::stepcounter++;

                // Call this function after our interval to do another step
                // but skip the init
                schedule("pdaDance::dance(1,0);", %interval);
            }
            else
            {
                // Adjust our counter since we haven't done a step this time
                $pdaDance::stepcounter--;

                // LOL, this is one way to avoid recursion; I'm amazed this doesn't
                // cause any problems.  My guess is the schedule pushes this onto a FIFO
                // queue and it doesn't get processed until this function returns...or
                // is each item on the schedule given its own thread, or the equivalent?

                // Call this function again to start retracing our footsteps
                schedule("pdaDance::dance(0,0);", 0);
            }
        }
        else // retracing our footsteps
        {
            if($pdaDance::stepcounter >= 0)
            {
                // Still retracing the steps -- we're getting the steps in reverse order
                // and doing the opposite action for each

                // Get the next step to retrace and determine which step will retrace it
                %step = pdaDance::getRetraceStep($pdaDance::stepsTaken[$pdaDance::stepcounter]);

                // Start retracing this step
                eval("pdaDance::Step" @ %step @ "(1);");

                // Use the same interval used for the "forward" version of this step
                %interval = $pdaDance::intervals[$pdaDance::stepcounter];

                // Stop retracing this step after our interval has passed
                schedule("pdaDance::Step" @ %step @ "(0);", %interval);

                // Retrace the next step next time (or start going forward again)
                $pdaDance::stepcounter--;

                // Call this function again after our interval to keep retracing
                schedule("pdaDance::dance(0,0);", %interval);
            }
            else
            {
                // Call this function again to do the init and start dancing "forward" again
                schedule("pdaDance::dance(1,1);", 0);
            }
        }
    }
}

function pda_dance::save()
{
    AddExportText("    // Start or stop dancin' :)");

    ExportBind("pdaDance();", "//bind(keyboard0, make, alt, \"d\", TO, \"pdaDance();\");");
    ExportBind("pdaDance_break", "//bind(keyboard0, break, alt, \"d\", TO, \"\", pdaDance_break);");

    AddExportText("");
    AddExportText("    // Start dancin' and bring up the commander screen");
    AddExportText("    // or stop dancin' and close the commander screen");

    ExportBind("pdaDance::commandScreen();", "//bind(keyboard0, make, alt, \"c\", TO, \"pdaDance::commandScreen();\");");
    ExportBind("pdaDance::commandScreen_break", "//bind(keyboard0, break, alt, \"c\", TO, \"\", pdaDance::commandScreen_break);");
}

WriterPrefs::save("pda_dance");

function pda_dance::init()
{
    needs("writer\\throw.cs", "writer\\save.cs 2.3");
}
Event::Attach(writerInit, "pda_dance::init();");

if(!$UsingWriterLoader) pda_dance::init();
